@@ -18,18 +18,14 @@
 #include <QGuiApplication>
 #include <QTimeZone>
 
+#include <KFormat>
 #include <KLocalizedString>
 
 #include "enums/delegatetype.h"
 #include "eventhandler.h"
 #include "events/pollevent.h"
-#include "linkpreviewer.h"
-#include "messagecontentmodel.h"
 #include "models/messagefiltermodel.h"
 #include "models/reactionmodel.h"
-#include "neochatroom.h"
-#include "neochatroommember.h"
-#include "readmarkermodel.h"
 #include "texthandler.h"
 
 using namespace Quotient;
@@ -72,6 +68,11 @@ MessageEventModel::MessageEventModel(QObject *parent)
     connect(this, &MessageEventModel::modelReset, this, [this]() {
         resetting = false;
     });
+
+    connect(NeoChatConfig::self(), &NeoChatConfig::ThreadsChanged, this, [this]() {
+        beginResetModel();
+        endResetModel();
+    });
 }
 
 NeoChatRoom *MessageEventModel::room() const
@@ -89,7 +90,6 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
         // HACK: Reset the model to a null room first to make sure QML dismantles
         // last room's objects before the room is actually changed
         beginResetModel();
-        m_readMarkerModels.clear();
         m_currentRoom->disconnect(this);
         m_currentRoom = nullptr;
         endResetModel();
@@ -97,6 +97,9 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
         // Don't clear the member objects until the model has been fully reset and all
         // refs cleared.
         m_memberObjects.clear();
+        m_contentModels.clear();
+        m_reactionModels.clear();
+        m_readMarkerModels.clear();
     }
 
     beginResetModel();
@@ -410,7 +413,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
             return DelegateType::ReadMarker;
         case TimeRole: {
             const QDateTime eventDate = data(index(m_lastReadEventIndex.row() + 1, 0), TimeRole).toDateTime().toLocalTime();
-            const KFormat format;
+            static const KFormat format;
             return format.formatRelativeDateTime(eventDate, QLocale::ShortFormat);
         }
         case SpecialMarksRole:
@@ -431,26 +434,25 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     const auto pendingIt = m_currentRoom->pendingEvents().crbegin() + std::min(row, timelineBaseIndex());
     const auto &evt = isPending ? **pendingIt : **timelineIt;
 
-    EventHandler eventHandler(m_currentRoom, &evt);
-
     if (role == Qt::DisplayRole) {
-        return eventHandler.getRichBody();
+        return EventHandler::richBody(m_currentRoom, &evt);
     }
 
     if (role == ContentModelRole) {
-        if (!evt.isStateEvent() && !evt.id().isEmpty()) {
-            return QVariant::fromValue<MessageContentModel *>(new MessageContentModel(m_currentRoom, &evt));
+        QString modelId;
+        if (!evt.id().isEmpty() && m_contentModels.contains(evt.id())) {
+            modelId = evt.id();
+        } else if (!evt.transactionId().isEmpty() && m_contentModels.contains(evt.transactionId())) {
+            modelId = evt.transactionId();
         }
-        if (evt.isStateEvent()) {
-            if (evt.matrixType() == QStringLiteral("org.matrix.msc3672.beacon_info")) {
-                return QVariant::fromValue<MessageContentModel *>(new MessageContentModel(m_currentRoom, &evt));
-            }
+        if (!modelId.isEmpty()) {
+            return QVariant::fromValue<MessageContentModel *>(m_contentModels.at(modelId).get());
         }
         return {};
     }
 
     if (role == GenericDisplayRole) {
-        return eventHandler.getGenericBody();
+        return EventHandler::genericBody(m_currentRoom, &evt);
     }
 
     if (role == DelegateTypeRole) {
@@ -473,7 +475,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     }
 
     if (role == HighlightRole) {
-        return eventHandler.isHighlighted();
+        return EventHandler::isHighlighted(m_currentRoom, &evt);
     }
 
     if (role == SpecialMarksRole) {
@@ -486,7 +488,11 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
             return pendingIt->deliveryStatus();
         }
 
-        if (eventHandler.isHidden()) {
+        if (EventHandler::isHidden(m_currentRoom, &evt)) {
+            return EventStatus::Hidden;
+        }
+
+        if (EventHandler::isThreaded(&evt) && EventHandler::threadRoot(&evt) != EventHandler::id(&evt) && NeoChatConfig::threads()) {
             return EventStatus::Hidden;
         }
 
@@ -494,16 +500,12 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     }
 
     if (role == EventIdRole) {
-        return eventHandler.getId();
+        return EventHandler::id(&evt);
     }
 
     if (role == ProgressInfoRole) {
         if (auto e = eventCast<const RoomMessageEvent>(&evt)) {
-#if Quotient_VERSION_MINOR > 8
             if (e->has<EventContent::FileContent>()) {
-#else
-            if (e->hasFileContent()) {
-#endif
                 return QVariant::fromValue(m_currentRoom->cachedFileTransferInfo(&evt));
             }
         }
@@ -514,20 +516,20 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
 
     if (role == TimeRole) {
         auto lastUpdated = isPending ? pendingIt->lastUpdated() : QDateTime();
-        return eventHandler.getTime(isPending, lastUpdated);
+        return EventHandler::time(&evt, isPending, lastUpdated);
     }
 
     if (role == SectionRole) {
         auto lastUpdated = isPending ? pendingIt->lastUpdated() : QDateTime();
-        return eventHandler.getTimeString(true, QLocale::ShortFormat, isPending, lastUpdated);
+        return EventHandler::timeString(&evt, true, QLocale::ShortFormat, isPending, lastUpdated);
     }
 
     if (role == IsThreadedRole) {
-        return eventHandler.isThreaded();
+        return EventHandler::isThreaded(&evt);
     }
 
     if (role == ThreadRootRole) {
-        return eventHandler.threadRoot();
+        return EventHandler::threadRoot(&evt);
     }
 
     if (role == ShowSectionRole) {
@@ -580,7 +582,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     }
 
     if (role == AuthorDisplayNameRole) {
-        return eventHandler.getAuthorDisplayName(isPending);
+        return EventHandler::authorDisplayName(m_currentRoom, &evt, isPending);
     }
 
     if (role == IsRedactedRole) {
@@ -592,11 +594,11 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     }
 
     if (role == MediaInfoRole) {
-        return eventHandler.getMediaInfo();
+        return EventHandler::mediaInfo(m_currentRoom, &evt);
     }
 
     if (role == IsEditableRole) {
-        return eventHandler.messageComponentType() == MessageComponentType::Text && evt.senderId() == m_currentRoom->localMember().id();
+        return MessageComponentType::typeForEvent(evt) == MessageComponentType::Text && evt.senderId() == m_currentRoom->localMember().id();
     }
 
     return {};
@@ -624,6 +626,9 @@ void MessageEventModel::createEventObjects(const Quotient::RoomEvent *event)
 
     auto eventId = event->id();
     auto senderId = event->senderId();
+    if (eventId.isEmpty()) {
+        eventId = event->transactionId();
+    }
     // A pending event might not have a sender ID set yet but in that case it must
     // be the local member.
     if (senderId.isEmpty()) {
@@ -634,6 +639,16 @@ void MessageEventModel::createEventObjects(const Quotient::RoomEvent *event)
         m_memberObjects[senderId] = std::unique_ptr<NeochatRoomMember>(new NeochatRoomMember(m_currentRoom, senderId));
     }
 
+    if (!m_contentModels.contains(eventId) && !m_contentModels.contains(event->transactionId())) {
+        if (!event->isStateEvent() || event->matrixType() == QStringLiteral("org.matrix.msc3672.beacon_info")) {
+            m_contentModels[eventId] = std::unique_ptr<MessageContentModel>(new MessageContentModel(m_currentRoom, eventId));
+        }
+    }
+
+    if (EventHandler::isThreaded(event) && !m_threadModels.contains(EventHandler::threadRoot(event))) {
+        m_threadModels[EventHandler::threadRoot(event)] = QSharedPointer<ThreadModel>(new ThreadModel(EventHandler::threadRoot(event), m_currentRoom));
+    }
+
     // ReadMarkerModel handles updates to add and remove markers, we only need to
     // handle adding and removing whole models here.
     if (m_readMarkerModels.contains(eventId)) {
@@ -693,4 +708,9 @@ bool MessageEventModel::event(QEvent *event)
     return QObject::event(event);
 }
 
+ThreadModel *MessageEventModel::threadModelForRootId(const QString &threadRootId) const
+{
+    return m_threadModels[threadRootId].data();
+}
+
 #include "moc_messageeventmodel.cpp"
